/***************************************************************************************************/
/*                                                                                                 */
/* file:          Controller.h                                                                     */
/*                                                                                                 */
/* source:        2015-2106, written by Adrian Kundert                                             */
/*                                                                                                 */
/* description:   The main class for the Self-Balanced PT project                                  */
/*                                                                                                 */
/***************************************************************************************************/

#ifndef CONTROLLER_H
#define CONTROLLER_H

#include "MPU_6050.h"
#include "RingBuffer.h"

#define SwVersion "0.0.1.0"

#define LED_PIN 13
#define READY_SW_PIN 5
#define LOADED_PIN 4

class Controller
{
public:
    Controller();
    void initialize();
    void process();
    void MPUinterrupt();
    void serialPrint();
private:
    bool serialReadisCR();
    
public:

private:
    MPU_6050 mpuTilt;   // first IMU sensor MPU-6050 for the tilt
    MPU_6050 mpuSteer;  // second IMU sensor MPU-6050 for the steer

    volatile unsigned long isrCnt;
    volatile bool MPUinterruptFlag;

    byte ReadySwCnt; bool ReadySw;
    const char ReadySwCntMax = 5;  // for debouncing
    
    byte Kp, Ki, Kd;
    long P, I, D, LastError;
    enum CtrlState {Idle=0, StandingUp=1, Up=2, Running=3, Balancing=4, Pivot=5};
    CtrlState State;
    byte BalancingCnt;
    
    char SpeedMotorLeft, SpeedMotorRight, DirMotorLeft, DirMotorRight, CmdSpeedMotor;
    float SteerAngle;
    
    RingBuffer rbuffer;
};

#endif //CONTROLLER_H


